//////////////////////////////////////////////////
// Wars Hauptdatei mit Grundlegenden Funktionen //
//////////////////////////////////////////////////

#include "StdHeaders.h"

//Globale Variablen
BITMAP *doubleBuffer = NULL, *multiBG = NULL, *creditPicture = NULL, *helppage1 = NULL, *helppage2 = NULL;
DATAFILE *datafile = NULL;
FONT *tahoma = NULL;
int curmode = GFX_AUTODETECT_WINDOWED, showFPS = 1;
int key_enter = 0, key_esc = 0;

//Timerfunktion und Variable
volatile int timerCounter = 0;
static void timerCounterUpdater()
{
	timerCounter++;
};
END_OF_STATIC_FUNCTION(timerCounterUpdater);

//Setzt den Timercounter auf 1 zurck
void reset()
{
	if(timerCounter > 1)
		timerCounter = 1;
};

//Wartet bis der timerCounter wieder erhht wurde
void wait(int time)
{
	while(timerCounter < time);
	timerCounter -= time;
};

//Funktion zur Fehlerausgabe
void fatalError(char *str)
{
	set_gfx_mode(GFX_TEXT, 0,0,0,0);
    allegro_message(str);
    exit(0);
};

//Initialisierungsfunktion
void init()
{
	//Allegro, Timer, Tastatur und Sound initialisieren
	allegro_init();
	if(install_timer() < 0)
		fatalError("Error009: Timer could not be initialised");
	if(install_keyboard() < 0)
		fatalError("Error007: No keyboarddriver found");
	if(install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL) < 0)
		fatalError("Error008: No sounddriver found");

	//Fenstertitel festlegen
	set_window_title("Wars - Polecatsoft");

	//Grafikmodus festlegen
	int depth = desktop_color_depth();
	if(!depth)
		depth = 16;
	set_color_depth(depth);
#ifdef ALLEGRO_STATICLINK
	curmode = GFX_AUTODETECT_FULLSCREEN;
#endif
	if(set_gfx_mode(curmode, 800, 600, 0, 0) < 0)
	{
		if(curmode == GFX_AUTODETECT_FULLSCREEN)
			curmode = GFX_AUTODETECT_WINDOWED;
		else
			curmode = GFX_AUTODETECT_FULLSCREEN;
		if(set_gfx_mode(curmode, 800, 600, 0, 0) < 0)
			Error001();
	};

	//Doublebuffer erstellen
	doubleBuffer = create_bitmap();

	multiBG = load_bitmap("background.tga");

    //Timer Funktion und Variable locken
	LOCK_FUNCTION(timerCounterUpdater);
	LOCK_VARIABLE(timerCounter);

    //Auf 36 fps einstellen
	install_int_ex(timerCounterUpdater, BPS_TO_TIMER(36));

	//Zeichenformat einstellen
	set_uformat(U_ASCII);

	//Rahmen um den Text verschwinden lassen
	text_mode(-1);

	//Schriftart laden
	datafile = load_datafile("font.dat");
	if(!datafile)
		Error005("font.dat");

	tahoma = (FONT*) datafile[0].dat;

#ifdef ALLEGRO_STATICLINK
	showFPS = 0;
#endif
};

//Speicher freigeben
void free_memory()
{
	destroy_bitmap(doubleBuffer);
	destroy_bitmap(multiBG);
	if(creditPicture)
		destroy_bitmap(creditPicture);
	if(helppage1)
	{
		destroy_bitmap(helppage1);
		destroy_bitmap(helppage2);
	};
	if(buyScreen)
		destroy_bitmap(buyScreen);
	if(mapSelectScreen)
		destroy_bitmap(mapSelectScreen);
	if(menuBackground)
		destroy_bitmap(menuBackground);
	if(bgbox)
		destroy_bitmap(bgbox);
	unload_datafile(datafile);
	for(int a = 0; a < 2; ++a)
		for(int i = 0; i < 7; ++i)
			if(sprites[a][i])
				destroy_bitmap(sprites[a][i]);

	mouse_uninstall();
};

//doubleBuffer auf den Bildschirm drucken und danach lschen
void show(int clearto)
{
    blit(doubleBuffer, screen);
	if(clearto >= 0)
		clear_to_color(doubleBuffer, clearto);
};


//Zeigt das Bild picture an
void title(BITMAP *titleBitmap, const int KEY_LOCK, const int MAX_LOCK)
{
	int counter = 0;

	reset();

	clear_keybuf();

	while(TRUE)
	{
		//Wenn der Spieler eine Taste gedrckt hat,
		//Oder 7 Sekunden vorbei sind,
		//Schleife beenden
		if((counter > KEY_LOCK && keypressed()) || MAX_LOCK > 0 && counter > MAX_LOCK)
			break;

		blit(titleBitmap, doubleBuffer);
		show();
		++counter;

		wait();
	};

	//Schwarzen Bildschirm erzeugen
	show();
};
//Zeigt as angegebene Bild "picture" an
void title(char *picture, const int KEY_LOCK, const int MAX_LOCK)
{
	//Bild laden
	BITMAP *titleBitmap = load_bitmap(picture);

	title(titleBitmap, KEY_LOCK, MAX_LOCK);

	//Speicher freigeben und Wert zurckgeben
	destroy_bitmap(titleBitmap);
};

//Hauptfunktion des Programms
int main(int argc, char **argv)
{
	init();
	mouse_init();

#ifdef ALLEGRO_STATICLINK
	if(argc < 2)
	{
		BITMAP *logo = create_bitmap();
		clear_to_color(logo, makecol(255,255,255));
		BITMAP *logoBuffer = load_bitmap("logo.tga");
		blit(logoBuffer, logo, 156, 36, 483, 540);
		destroy_bitmap(logoBuffer);
		title(logo,  16, 252);
		destroy_bitmap(logo);
		title("title.tga", 16, 252);
	};
#endif

	MIDI *music = load_midi("midi/menu.mid");
	if(!music)
		Error011("midi/menu.mid");

	while(TRUE)
	{
		if(midi_pos < 0)
			play_midi(music, 1);
		if(argc > 1)
		{
			mapSelect(argv[1]);
			break;
		};
		int i = menu();
		if(!i
#ifdef ALLEGRO_STATICLINK
			&& yesno("Do you want to quit?")
#endif
			)
			break;

		char fileName[256] = "maps/MicroWar.war";
		switch(i)
		{
		case 1:
			if(!file_select_ex("Please select a mapfile to load.", fileName, "war;/-d", 256, SCREEN_W, SCREEN_H))
				continue;
			else
				mapSelect(fileName);
			break;
		case 2:
			if(!helppage1)
			{
				helppage1 = create_bitmap();
				helppage2 = create_bitmap();

				blit(multiBG, helppage1);
				blit(multiBG, helppage2);

				BITMAP *helpBuffer = load_bitmap("helppage1.tga");
				blit(helpBuffer, helppage1, 59, 74, 690, 482);
				destroy_bitmap(helpBuffer);
				helpBuffer = load_bitmap("helppage2.tga");
				blit(helpBuffer, helppage2, 35, 75, 678, 481);
				destroy_bitmap(helpBuffer);
			};
			title(helppage1, 18, 0);
			title(helppage2, 32, 0);
			break;
		case 3:
			if(!creditPicture)
			{
				creditPicture = create_bitmap();
				blit(multiBG, creditPicture);
				BITMAP *credits = load_bitmap("creditspage.tga");
				blit(credits, creditPicture, 163, 124, 475, 299);
				destroy_bitmap(credits);
			};
			title(creditPicture, 18, 0);
			break;
		};
	};

	destroy_midi(music);
	free_memory();
	return 0;
};
END_OF_MAIN();
