///////////////////////////////////////////////////////
// Mapklasse, verwaltet Informationen ber die Karte //
///////////////////////////////////////////////////////

#include "map.h"

void map::drawTile(int x, int y, int tile)
{
	blit(tiles, dest, tile * tileWidth, 0, x, y, tileWidth, tileHeight);
};

void map::create(int w, int h)
{
	this->w = w;
	this->h = h;
	this->size = w * h;
	this->data = new char[size];
	this->dest = NULL;
	this->visibleTilesX = 0;
	this->visibleTilesY = 0;
};

map::map(int w, int h)
{
	create(w, h);
};

map::map(int w, int h, char *mapString, char *tileStr)
{
	create(w, h);
	for(int a = 0; a < size; ++a)
	{
		for(int tile = 0; tileStr[tile]; ++tile)
		{
			if(mapString[a] == tileStr[tile])
			{
				data[a] = tile;
				break;
			};
		};
	};
};

map::map(char *mapname)
{
	ifstream readfile(mapname, ios::binary);
	if(readfile.fail())
		Error010(mapname);
	char wAndH[2];
	readfile.read(&wAndH[0], 1);
	readfile.read(&wAndH[1], 1);
	create(wAndH[0], wAndH[1]);
	for(int i = 0; i < size; ++i)
	{
		char temporary;
		readfile.read(&temporary, 1);
		data[i] = temporary;
	};
	readfile.close();
};

map::~map()
{
	if(size > 0)
		delete [] data;
};

void map::setTargetBitmap(BITMAP *dest)
{
	this->dest = dest;

	visibleTilesX = 20;
	visibleTilesY = 11;

	maxX = w - 20;
	maxY = h - 11;
};

int map::getTileAt(int tileX, int tileY)
{
	int pos = tileX + tileY * w;
	if(pos >= 0 && pos < size)
		return data[pos];
	return -1;
};

void map::setTileAt(int tileX, int tileY, int tile)
{
	int pos = tileX + tileY * w;
	if(pos >= 0 && pos < size)
		data[pos] = (char) (tile & 0xff);
};

void map::setTiles(BITMAP *tiles, int tw, int th)
{
	this->tiles = tiles;
	tileWidth   = tw;
	tileHeight  = th;
};

void map::draw(int wx, int wy)
{
	int pos = 0;

	wx = MID(0, wx, maxX);
	wy = MID(0, wy, maxY);

	for(int y = 0; y < visibleTilesY; ++y)
	{
		pos = wx + (y + wy) * w;
		for(int x = 0; x < visibleTilesX + 1; ++x)
		{
			drawTile(x * tileWidth, y * tileHeight, data[pos]);
			++pos;
		};
	};
};

int map::getTileWidth()
{
	return tileWidth;
};

int map::getTileHeight()
{
	return tileHeight;
};

BITMAP *map::getTargetBitmap()
{
	return dest;
};

int map::isWalkable(int tileX, int tileY, int type)
{
	if(tileX < 0 || tileX >= w || tileY < 0 || tileY >= h)
		return 0;
	int tile = getTileAt(tileX, tileY);
	int points = 1;
	if(type == INFANTRY && ((tile >= 8 && tile <= 13) || tile == 22))
			points = 2;
	else if(type == KAVALLRY)
	{
		if(tile == 22 || (tile >= 8 && tile <= 13))
			points = 0;
		else if(tile == 15)
			points = 2;
	}
	else if(type == ARTILLRY)
	{
		if(tile == 22 || (tile >= 8 && tile <= 13))
			points = 0;
		else if(tile == 14)
			points = 2;
		else if(tile == 15)
			points = 3;
	};

	return points;
};

int map::getDefense(int tileX, int tileY)
{
	int tile = getTileAt(tileX, tileY);
	int defense = 0;
	if(tile < 14)
		return defense;

	if(tile > 22)
		defense = 5;
	else if(tile == 14)
		defense = 1;
	else if(tile == 15)
		defense = 2;
	else if(tile >= 16 && tile <= 21)
		defense = 3;
	else if(tile == 22)
		defense = 4;

	return defense;
};

/*int map::getDefense(int tileX, int tileY, int curUnitTeam)
{
	int tile = getTileAt(tileX, tileY);
	int defense = 0;
	if(tile < 14)
		return defense;

	if(tile == 23)
	{
		if(!curUnitTeam)
			defense = 5;
		else
			defense = 0;
	}
	else if(tile == 24)
	{
		if(curUnitTeam)
			defense = 5;
		else
			defense = 0;
	}
	else if(tile == 14)
		defense = 1;
	else if(tile == 15)
		defense = 2;
	else if(tile >= 16 && tile <= 21)
		defense = 3;
	else if(tile == 22)
		defense = 4;

	return defense;
};*/

char *map::getName(int tileX, int tileY)
{
	int tile = getTileAt(tileX, tileY);
	if(tile < 6)
		return "Street";
	if(tile < 8)
		return "Bridge";
	if(tile < 14)
		return "River";
	if(tile < 15)
		return "Grassland";
	if(tile < 16)
		return "Forest";
	if(tile < 19)
		return "Village";
	if(tile < 22)
		return "Barracks";
	if(tile < 23)
		return "Mountain";
	if(tile < 25)
		return "Castle";

	return "UNKNOWN";
};

int map::getMaxX()
{
	return maxX;
};

int map::getMaxY()
{
	return maxY;
};

int map::getHeight()
{
	return h;
};

int map::getWidth()
{
	return w;
};

void map::addMoney(int *money, int player, int moneyPerHouse)
{
	for(int i = 0; i < size; ++i)
	{
		if(data[i] == 23 + player || data[i] == 20 + player || data[i] == 17 + player)
			*money += moneyPerHouse;
	};
};
